version: str = '0.0.11.8'
""" Module fenv main """
from fenv.customizes.colors import Colors
from fenv.assets.commands import Commands
from fenv.env_all import EnvAll
from fenv.manage_file import (
    CreateFileBaseAndUpdate,
    OnlyVirtualEnv,
    InstallModule,
    UninstallModule,
    Cleanup,
)

from argparse import ArgumentParser
import os
import platform
import fnmatch


colors = Colors()
notice = colors.notice()

env_above = EnvAll()
env_directory = env_above.get_env_name()
root_directory = env_above.get_root_dir_name()


def find_dir_env() -> str:
    """
    It prints the contents of the current directory

    Return:
        None
    """
    for i in os.listdir("."):
        print(i)


def setup_parse():
    """
    It takes a list of strings, and returns a dictionary of the form {'name': 'value'}
    :return: The return value is a Namespace object.
    """

    parser = ArgumentParser(add_help=False)
    title = parser.add_argument_group(title="Usage")
    title.description = "fenv <command>"

    subparsers = parser.add_subparsers(title="Commands", dest="command", metavar="")

    new_comd = subparsers.add_parser("new", help="Create a new project")
    new_comd.add_argument(
        "new", type=str, help="The name of the project", nargs="?", default=None
    )

    install_cmd = subparsers.add_parser(
        "install",
        help="Install the package and install the package via requirements.txt",
        usage=f"{colors.NAVY}fenv install {colors.NAVY}<package_name>{colors.ENDC} or {colors.HOT_PINK}fenv install {colors.ENDC}",
    )
    install_cmd.add_argument(
        "install",
        type=str,
        help="Install packages of the project packages",
        nargs="?",
        default=None,
    )

    uninstall_cmd = subparsers.add_parser(
        "uninstall",
        help="Uninstall packages",
        usage=f"{colors.HOT_PINK}fenv uninstall <package_name>{colors.ENDC}",
    )
    uninstall_cmd.add_argument(
        "uninstall",
        type=str,
        help="Uninstall packages of the project packages",
        nargs="?",
        default=None,
    )

    update_cmd = subparsers.add_parser(
        "update",
        help="Package to file requirements.txt update furthermore, update the readme.md file's tree path.",
        usage=f"{colors.HOT_PINK}fenv update{colors.ENDC}",
    )

    onlyenv_cmd = subparsers.add_parser(
        "onlyenv",
        help=f"Create only virtualenv and no create base file",
        usage=f"{colors.HOT_PINK}fenv onlyenv{colors.ENDC}",
    )

    clean_cmd = subparsers.add_parser(
        "clean", help="Clean delete all packages in requirements.txt out"
    )
    test_cmd = subparsers.add_parser("test", help="test")

    general_group = parser.add_argument_group(title="General Options")
    general_group.add_argument(
        "-h", "--help", action="help", help="Show this help message and exit"
    )
    general_group.add_argument(
        "-v", "--version", action="store_true", help="check version fenv"
    )

    args = parser.parse_args()

    return args


def check_command(args):
    """
    It checks the command that the user has entered and then runs the appropriate function

    Args:
        args (str): The arguments passed to the script

    Example:
        ```py
        check_command("new" or "install" or "uninstall" or "update" or "onlyenv")
        ```
    Return:
        None
    """
    if args.__dict__["command"] == "new":
        try:
            CreateFileBaseAndUpdate(args.new, "create").procress_only_create_project()
        except TypeError as err:
            print(
                f"Maybe you forgot to enter the name of the folder? For example: {colors.LIGHTGREEN_EX}fenv new{colors.NAVY} <project_folder>{colors.ENDC}"
            )

    elif args.__dict__["command"] == "install":
        InstallModule(
            args
        ).install_package_only() if args.install != None else InstallModule(
            args
        ).install_package_all()

    elif args.__dict__["command"] == "uninstall":
        UninstallModule(args).process_run() if args.uninstall != None else print(
            f"Maybe you forgot to put the name of the package to uninstall? "
            f"For example: {colors.LIGHTGREEN_EX}fenv uninstall{colors.OKBLUE} <package_name>{colors.ENDC}"
        )
    elif args.__dict__["command"] == "update":
        CreateFileBaseAndUpdate(
            root_directory, "update"
        ).process_create_base_file_and_update()
        print(notice + "Updated tree path to readme.md")
        os.system("pip freeze > requirements.txt")
        print(notice + "Updated module all to requirements.txt")
    elif args.__dict__["command"] == "onlyenv":
        OnlyVirtualEnv().run_process()

    elif args.__dict__["command"] == "clean":
        question = input(
            "Do you want to delete all packages in requirements.txt out? (y/n) "
        ).lower()
        if question == "y" or question == "":
            Cleanup().remove_lib_not_default_in_env()
    elif args.__dict__["command"] == "test":
        print("D:D:D:D:D:D:D:D")


def main():
    """
    It takes the arguments from the command line and passes them to the create_project_all function
    Return:
        None
    """
    args = setup_parse()

    print(
        f"⏩ {colors.LIGHTMAGENTA_EX}Hello,Fenv {colors.POWDER_BLUE}[{colors.MINT_GREEN}v{version}{colors.POWDER_BLUE}]{colors.ENDC}🫡\n".center(
            40, "-"
        )
    ) if args.__dict__["command"] == None else None
    check_command(args)



""" Module about passing folder names """
import fnmatch
import os
import configparser
import platform
import glob


class EnvAll:
    def __init__(self, **kwargs):
        self.kwargs = kwargs
        self.config = configparser.ConfigParser()

    def get_env_name(self):
        """It's a function that checks if the environment directory exists.
        Return:
            [] : empyt list
        """
        self.folder_name = "env*"
        return (
            fnmatch.filter(os.listdir("."), self.folder_name)
            if fnmatch.filter(os.listdir("."), self.folder_name) == []
            else str(fnmatch.filter(os.listdir("."), self.folder_name)[0])
        )

    def get_root_dir_name(self):
        """It's a function that checks if the root directory exists.

        Return:
            str : root directory
        """
        return os.path.basename(os.path.abspath("."))

    def create_lib_default_env(self):
        """It's a function that creates the env.
        Return:
            str : env name
        """
        self.data_list = []

        if platform.system() == "Windows":
            path_lib_all = self.get_path_lib_all()
            for i in os.listdir(path_lib_all):
                self.data_list.append(i)
                self.data_string = ",".join(self.data_list)
                self.config["fenv"] = {"default_lib": self.data_string}
                with open(rf"{self.get_env_name()}\fenv.cfg", "w") as configfile:
                    self.config.write(configfile)
        elif platform.system() == "Linux":
            path_lib_all = self.get_path_lib_all()
            for i in os.listdir(path_lib_all):
                self.data_list.append(i)
                self.data_string = ",".join(self.data_list)
                self.config["fenv"] = {
                    "default_lib": self.data_string,
                }
                with open(rf"{self.get_env_name()}/fenv.cfg", "w") as configfile:
                    self.config.write(configfile)

    def get_lib_default_env(self):
        """
        It reads a config file, gets a string, splits it into a list, and returns the list
        Return:
            data_list : list
        """
        self.config.read(rf"{self.get_env_name()}/fenv.cfg")

        self.data_string = self.config.get("fenv", "default_lib")

        self.data_list = self.data_string.split(",")
        return self.data_list

    def get_path_lib_all(self):
        """
        It's a function that checks if the environment directory exists.
        Return:
            path_lib_all : str
        """
        if platform.system() == "Windows":
            return f"{self.get_env_name()}\Lib\site-packages"
        elif platform.system() == "Linux":
            return "".join(
                glob.glob(f"{self.get_env_name()}/lib/python*/site-packages")
            )



""" Module manage create file """
import os
import shutil
import subprocess
import platform
import re
import random
import fnmatch
from fenv.customizes.colors import Colors
from fenv.assets.commands import Commands
from fenv.env_all import EnvAll


class CreateFileBaseAndUpdate:
    def __init__(self, name, state):
        self.name = name
        self.state = state
        self.colors = Colors()
        self.notice = Colors().notice()
        self.commands = Commands()
        self.env_directory = EnvAll().get_env_name()
        self.root_directory = EnvAll().get_root_dir_name()

    def create_file_main_py(self):
        """
        Create a file main.py and write a function called main() inside of it
        """
        self.file_path = "main.py"
        with open(self.file_path, "w") as f:
            f.write(self.commands.get_main_py())
        os.chmod(self.file_path, 0o777)
        print(self.notice + f'Successfully created the file "{self.file_path}"')

    def generate_tree(self):
        self.output = ""
        for root, dirs, files in os.walk("."):
            level = root.replace(".", "").count(os.sep)
            if level == 0:
                indent = "    " * (level + 1) + "│   "
                subindent = "    " * (level + 2)

                for d in dirs:
                    dir_path = os.path.join(root, d)
                    self.output += f"{subindent}{'└──'}{d}/\n"
                    if d == ".vscode":
                        self.output += f"{subindent}{subindent}{'└──'}settings.json\n"
                    if d == self.env_directory:
                        for i in ["Lib/", "Scripts", ".gitignore", "pyvenv.cfg"]:
                            self.output += f"{subindent}{subindent}{'└──'} {i}\n"

                for f in files:
                    file_path = os.path.join(root, f)
                    self.output += f"{subindent}{'└──'}{f}\n"

        return self.output

    def create_file_readme_md(self):
        """
        It creates a file called readme.md and writes the markdown text to it
        """
        markdown_path = "readme.md"
        markdown = self.commands.get_readme_md()
        with open(markdown_path, "w", encoding="utf-8") as f:
            f.write(
                markdown.format(
                    self.name, self.name, self.name, self.name, self.generate_tree()
                )
            )
        os.chmod(markdown_path, 0o777)
        print(self.notice + f'Successfully created the file "{markdown_path}"')

    def create_file_freeze(self):
        """
        It creates a file called "requirements.txt" and writes the string "black" to it
        """
        module_base = self.commands.get_requirements_txt()
        with open("requirements.txt", "w") as f:
            f.write(module_base)
        os.chmod("requirements.txt", 0o777)
        print(self.notice + f'Successfully created the file "requirements.txt"')

    def create_file_gitignore(self):
        """
        It creates a file called .gitignore and writes the string "*.pyc" to it
        """
        with open(".gitignore", "w") as f:
            f.write(f"*.pyc\n/{self.env_directory}")
        os.chmod(".gitignore", 0o777)
        print(self.notice + f'Successfully created the file ".gitignore"')

    def update_file_readme_md(self):
        """
        It update a file called readme.md and writes the markdown text to it
        """
        markdown_path = "readme.md"
        with open(markdown_path, "r", encoding="utf-8") as f:
            data = f.readlines()

        for i, v in enumerate(data):
            if "<!--- Start Tree --->" in v:
                first = i

            if "<!--- End Tree --->" in v:
                last = i

        data = data[: first + 1] + data[last:]

        for i, v in enumerate(data):
            if "<!--- Start Tree --->" in v:
                data[i] = self.commands.get_update_tree_path().format(
                    self.name, self.generate_tree()
                )
        with open(markdown_path, "w", encoding="utf-8") as f:
            f.writelines(data)
        os.chmod(markdown_path, 0o777)

    def create_virtualenv(self):
        """
        It creates a virtual environment with the name you pass to it

        Args:
          virtual_env_name (str): The name of the virtual environment you want to create.

        Example:
            ```py
            create_virtualenv("virtual_env_name")
            ```
        Return:
            None
        """
        if os.path.exists(self.name):
            os.chdir(self.name)
            print(f"createing virtualenv env_{self.name}...")
            os.system(f"virtualenv env_{self.name}")
            EnvAll().create_lib_default_env()

        print(self.notice + f'Successfully created the virtualenv "{self.name}"')

    def create_setting_vscode(self):
        """
        It creates a file settings.json inside the virtual environment
        Args:
        env_path (str): The path to the virtual environment
        Example:
            ```py
            create_setting_vscode("env_path")
            ```
        Return:
            None
        """

        text_vscode = """{{"python.formatting.provider": "black","python.pythonPath": "{name_env}","editor.formatOnSave": true,}}"""
        os.makedirs(os.path.dirname(".vscode/settings.json"), exist_ok=True)
        with open(".vscode/settings.json", "w", encoding="utf-8") as f:
            f.write(text_vscode.format(name_env=self.name))
        print(self.notice + f"Successfully created the .vscode/settings.json")

    def run_install_module_base(self):
        """
        It installs the base Python modules

        :param env: The environment object
        """

        if platform.system() == "Windows":
            os.system(
                f".\env_{self.name}\Scripts\python.exe -m pip install -r requirements.txt"
            )
            print(self.notice + f'Successfully installed module in "requirements.txt"')
            os.system(
                f".\env_{self.name}\Scripts\python.exe -m pip freeze > requirements.txt"
            )
            print(self.notice + f'Successfully updated the file "requirements.txt"')
            os.system(
                f".\env_{self.name}\Scripts\python.exe -m pip install --upgrade pip"
            )

        elif platform.system() == "Linux":
            os.system(
                f"bash -c 'source env_{self.name}/bin/activate  && pip install -r requirements.txt'"
            )
            print(self.notice + f'Successfully installed module in "requirements.txt"')
            os.system(
                f"bash -c 'source env_{self.name}/bin/activate  && pip freeze > requirements.txt'"
            )
            print(self.notice + f'Successfully updated the file "requirements.txt"')
            os.system(
                f"bash -c 'source env_{self.name}/bin/activate  && pip install --upgrade pip'"
            )

    def create_folder(self):
        """
        It creates a folder with the name of the argument passed to it

        Example:
            ```py
            create_folder("project_name")
            ```
        Return:
            1 : if has folder already
        """
        try:
            os.mkdir(self.name)
        except FileExistsError:
            print(self.notice + f"{self.name} already exists.")
            return 1
        else:
            print(self.notice + f'Successfully created the directory "{self.name}"')

    def process_create_base_file_and_update(self):
        """
        If the state is create, create the files main.py, freeze.py, gitignore.py, and readme.md. If the
        state is update, update the file readme.md
        """
        if self.state == "create":
            self.create_file_main_py()
            self.create_file_freeze()
            self.create_file_gitignore()
            self.create_file_readme_md()
            self.update_file_readme_md()
        elif self.state == "update":
            self.update_file_readme_md()

    def procress_only_create_project(self):
        """
        It creates a virtual environment and a vscode settings file.
        """
        if self.create_folder() != 1:
            self.create_virtualenv()
            self.create_setting_vscode()
            self.process_create_base_file_and_update()
            self.run_install_module_base()


class OnlyVirtualEnv:
    """Module create only env"""

    def __init__(self):
        self.colors = Colors()
        self.notice = Colors().notice()
        self.commands = Commands()
        self.env_directory = EnvAll().get_env_name()
        self.root_directory = EnvAll().get_root_dir_name()

    def create_name_env_auto(self) -> str:
        """
        > It creates a random name for the environment
        :return: A string
        """
        self.name_ = random.choice(["samai", "danai"])
        self.middle_ = random.choice("_#")
        self.no_ = random.randint(0, 100)
        return f"{self.name_}{self.middle_}{self.no_}"

    def create_name_env(self) -> str:
        """
        If the user enters a name for the virtualenv, the function will check if the name is in English
        only, if it is, it will return the name, if not, it will return a name automatically
        :return: The name of the virtual environment.
        """
        self.name = input(
            "Enter a name for the virtualenv (english only) , leave it blank to create it automatically: "
        ).replace(" ", "_")

        if bool(re.match("^[A-Za-z0-9_#]+$", self.name)):
            return "{}".format(self.name[:10])
        else:
            return self.create_name_env_auto()

    def create_virtualenv_not_change_dir(self):
        """
        It creates a virtual environment with the name you pass to it

        Args:
          virtual_env_name (str): The name of the virtual environment you want to create.

        Example:
            ```py
            create_virtualenv("virtual_env_name")
            ```
        Return:
            None
        """
        if not os.path.exists(self._name_env):
            print(f"virtualenv env_{self._name_env}")
            os.system(f"virtualenv env_{self._name_env}")
            EnvAll().create_lib_default_env()

        print(self.notice + f'Successfully created the virtualenv "{self._name_env}"')

    def run_process(self):
        """
        It creates a virtual environment and a vscode settings file.
        """
        self._name_env = self.create_name_env()
        print(
            f"your env name is `{self.colors.LIGHTGREEN_EX}{self._name_env}{self.colors.ENDC}`"
        )
        self.create_virtualenv_not_change_dir()
        CreateFileBaseAndUpdate(
            os.path.basename(os.getcwd()), ""
        ).create_setting_vscode()


class InstallModule:
    """Module install module"""

    def __init__(self, arg=None) -> None:
        self.package_name = arg
        self.colors = Colors()
        self.notice = Colors().notice()
        self.commands = Commands()
        self.env_directory = EnvAll().get_env_name()
        self.root_directory = EnvAll().get_root_dir_name()

    def install_required_package(self):
        """
        It checks if the platform is Windows, if it is, it runs the command:

        <code>.\{self.env_directory}\Scripts\python.exe -m pip install
        {self.package_name.install}</code>

        The problem is that it doesn't work
        """
        print(
            f"{self.notice} Installing {self.colors.PURPLE}{self.package_name.install}{self.colors.ENDC}{self.colors.SEA_GREEN}"
        )
        try:
            if platform.system() == "Windows":
                os.system(
                    rf".\{self.env_directory}\Scripts\python.exe -m pip install {self.package_name.install}"
                )
                print(
                    f"{self.notice}Successfully installed {self.colors.PURPLE}{self.package_name.install}{self.colors.ENDC}"
                )
            elif platform.system() == "Linux":
                os.system(
                    f"bash -c 'source {self.env_directory}/bin/activate  && pip install {self.package_name.install}'"
                )
                print(
                    f"{self.notice} Successfully installed {self.colors.PURPLE}{self.package_name.install}{self.colors.ENDC}"
                )

        except TimeoutError as e:
            print(e)

    def add_module_to_txt(self):
        """
        It takes the argument from the command line and adds it to the requirements.txt file
        Args:
            args (str): The arguments passed to the script

        Example:
            ```py
            add_module_to_txt("package_name")
            ```
        Return:
            None

        """
        if self.env_directory:
            if platform.system() == "Windows":
                os.system(
                    f".\{self.env_directory}\Scripts\python.exe -m pip freeze > requirements.txt"
                )
            elif platform.system() == "Linux":
                os.system(
                    f"bash -c 'source {self.env_directory}/bin/activate  && pip freeze > requirements.txt'"
                )
            print(
                self.notice
                + f'Successfully module {self.colors.PURPLE}{self.package_name.install}{self.colors.ENDC} added to "{self.colors.SEA_GREEN}requirements.txt{self.colors.ENDC}"'
            )

    def install_package_only(self):
        """
        It tries to install a package, if it fails, it prints a message

        Args:
        args (str): The arguments passed to the command.
        Example:
            ```py
            install_package_only("package_name")
            ```
        Return:
            None
        """
        try:
            if self.env_directory:
                self.install_required_package()
                self.add_module_to_txt()
            else:
                while True:
                    response = input(
                        "We couldn't find the fenv virtual environment. Would you like to set up a new one? (y/n): "
                    )
                    if (
                        response.lower() == "y"
                        or response.lower() == "yes"
                        or response.lower() == ""
                    ):
                        OnlyVirtualEnv().run_process()
                        folder_name = "env*"
                        folder_name_env = str(
                            fnmatch.filter(os.listdir("."), folder_name)[0]
                        )
                        self.install_required_package()
                        break
                    elif response.lower() == "n":
                        self.install_required_package()
                        break
        except AttributeError as err:
            print(
                self.colors.LIGHTGREEN_EX
                + "An error was encountered, it could not be installed."
                + self.colors.ENDC
            )

    def install_package_all(self):
        """
        install all packages in requirements.txt file using pip install -r requirements.txt
        """

        folder_name = "env*"
        folder_name_env = (
            fnmatch.filter(os.listdir("."), folder_name)
            if fnmatch.filter(os.listdir("."), folder_name) == []
            else str(fnmatch.filter(os.listdir("."), folder_name)[0])
        )
        requirements_file = "requirements.txt"

        def install_package_follow_env(folder_name_env):
            if platform.system() == "Windows":
                os.system(
                    f".\{folder_name_env}\Scripts\python.exe -m pip install -r requirements.txt"
                )
            elif platform.system() == "Linux":
                os.system(
                    f"bash -c 'source {folder_name_env}/bin/activate && pip install -r requirements.txt'"
                )

        def run_install_main(folder_name_env):
            if folder_name_env:
                print(
                    f"Found directory  `{self.colors.LIGHTGREEN_EX}{folder_name_env}{self.colors.ENDC}`"
                )
                print(
                    f"Installing modules with  `{self.colors.LIGHTGREEN_EX}{folder_name_env}{self.colors.ENDC}`"
                )
                install_package_follow_env(folder_name_env)
                print(
                    self.notice
                    + f'Successfully installed module from "requirements.txt"'
                )
            else:
                while True:
                    response = input(
                        "We couldn't find the fenv virtual environment. Would you like to set up a new one? (y/n): "
                    )
                    if (
                        response.lower() == "y"
                        or response.lower() == "yes"
                        or response.lower() == ""
                    ):
                        OnlyVirtualEnv().run_process()
                        folder_name = "env*"
                        folder_name_env = str(
                            fnmatch.filter(os.listdir("."), folder_name)[0]
                        )
                        print(
                            f"Installing modules with  `{self.colors.LIGHTGREEN_EX}{folder_name_env}{self.colors.ENDC}`"
                        )
                        install_package_follow_env(folder_name_env)
                        print(
                            self.notice
                            + f'Successfully installed modules from "requirements.txt"'
                        )
                        break
                    elif response.lower() == "n":
                        os.system(f"pip install -r requirements.txt")
                        break

        if requirements_file in os.listdir("."):
            run_install_main(folder_name_env)
        else:
            print(
                f"Maybe you forgot to put the name of the package to ininstall? for example `{self.colors.LIGHTGREEN_EX}fenv ininstall{self.colors.OKBLUE} <package_name>{self.colors.ENDC}` \nOr you can use `{self.colors.LIGHTGREEN_EX}fenv ininstall{self.colors.ENDC}` alone. But there must be {self.colors.FAIL}{requirements_file}{self.colors.ENDC} in the current directory"
            )


class UninstallModule:
    def __init__(self, arg) -> None:
        self.package_name = arg
        self.colors = Colors()
        self.notice = Colors().notice()
        self.commands = Commands()
        self.env_directory = EnvAll().get_env_name()
        self.root_directory = EnvAll().get_root_dir_name()

    def cmd_uninstall_package(self):
        """
        It uninstalls a package from the virtual environment

        Args:
            args (str): The arguments passed to the command
        Example:
            ```py
            cmd_uninstall_package("package_name")
            ```
        Return:
            None
        """

        try:
            package_dependency_list = self.pip_show_to_dict()["Requires"].split(", ")
            print(self.notice + f"Uninstalling...{self.colors.ORANGE}")
            if platform.system() == "Windows":
                os.system(
                    f".\{self.env_directory}\Scripts\python.exe -m pip uninstall {self.package_name.uninstall} -y"
                )
                for i in package_dependency_list:
                    os.system(
                        f".\{self.env_directory}\Scripts\python.exe -m pip uninstall {i} -y"
                    )

            elif platform.system() == "Linux":
                os.system(
                    f"bash -c 'source {self.env_directory}/bin/activate && pip uninstall {self.package_name.uninstall} -y'"
                )
                for i in package_dependency_list:
                    os.system(
                        f"bash -c 'source {self.env_directory}/bin/activate && pip uninstall {i} -y'"
                    )
            print(
                self.notice
                + f"Successfully uninstalled module {self.colors.LIGHTGREEN_EX}{self.package_name.uninstall}{self.colors.ENDC}"
            )
        except TimeoutError:
            print(TimeoutError)

    def remove_module_exit_txt(self):
        try:
            # os.system(f"pip freeze > requirements.txt")
            if platform.system() == "Windows":
                os.system(
                    f".\{self.env_directory}\Scripts\python.exe -m pip freeze > requirements.txt"
                )
            elif platform.system() == "Linux":
                os.system(
                    f"bash -c 'source {self.env_directory}/bin/activate && pip freeze > requirements.txt'"
                )
            print(
                self.notice
                + f'Successfully uninstalled module {self.colors.LIGHTGREEN_EX}{self.package_name.uninstall}{self.colors.ENDC} exit from "{self.colors.ORCHID}requirements.txt{self.colors.ENDC}"'
            )
        except Exception as e:
            print(f"Error: {e}")

    def process_run(self):
        """
        "A function that is called when the user runs the command "uninstall"."

        The first line of the function is a docstring. It's a string that describes what the function does.
        It's a good idea to include a docstring for every function you write

        Args:
        args (str): The arguments passed to the command
        Example:
            ```py
            process_run("package_name")
            ```
        Return:
            None
        """
        try:
            self.cmd_uninstall_package()
            self.remove_module_exit_txt()
        except AttributeError as err:
            print(err, "An error was encountered, it could not be uninstalled.")

    def pip_show_to_dict(self):
        result = {}
        try:
            # script
            if platform.system() == "Windows":
                output = subprocess.check_output(
                    f".\\{self.env_directory}\\Scripts\\python.exe -m pip show {self.package_name.uninstall}",
                    shell=True,
                ).decode("utf-8")
            elif platform.system() == "Linux":
                output = subprocess.check_output(
                    f"bash -c 'source {self.env_directory}/bin/activate && pip show {self.package_name.uninstall}'",
                    shell=True,
                ).decode("utf-8")
            for line in output.splitlines():
                match = re.match(r"^([^:]+):\s*(.*)$", line)
                if match:
                    key = match.group(1)
                    value = match.group(2)
                    result[key] = value
            return result
        except subprocess.CalledProcessError:
            print(
                f"{self.colors.ENDC}You have not installed the {self.colors.ORANGE}{self.package_name.uninstall}{self.colors.ENDC} package."
            )
            exit()


class Cleanup(CreateFileBaseAndUpdate):
    """Module for clean packages"""

    def __init__(self) -> None:
        self.colors = Colors()
        self.notice = Colors().notice()
        self.commands = Commands()
        self.path_lib_all = EnvAll().get_path_lib_all()
        self.lib_default_env = EnvAll().get_lib_default_env()
        self.env_name = EnvAll().get_env_name()

    def remove_lib_not_default_in_env(self):
        """
        Removes all the libraries that are not in the default environment
        """
        data_lib_all = []
        for item in os.listdir(self.path_lib_all):
            data_lib_all.append(item)

        diff_list_1 = set(data_lib_all) - set(self.lib_default_env)
        diff_list_2 = set(self.lib_default_env) - set(data_lib_all)
        result = diff_list_1.union(diff_list_2)
        os.chdir(self.path_lib_all)
        if len(result) > 0:
            for item in result:
                if os.path.isdir(item):
                    shutil.rmtree(item)
                    print(
                        self.notice
                        + f"{self.colors.SALMON}{item}{self.colors.ENDC} has been removed"
                    )
                else:
                    os.remove(item)
                    print(
                        self.notice
                        + f"{self.colors.SALMON}{item}{self.colors.ENDC} has been removed"
                    )
        os.chdir("../../..")
        print(
            self.notice
            + f"{self.colors.SKY_BLUE}All the libraries have been removed.{self.colors.ENDC}"
        )
        if platform.system() == "Windows":
            os.system(
                f".\env_{self.env_name }\Scripts\python.exe -m pip freeze > requirements.txt"
            )
            print(
                self.notice
                + f'{self.colors.SKY_BLUE}Successfully updated the file "requirements.txt"{self.colors.ENDC}'
            )

        elif platform.system() == "Linux":
            os.system(
                f"bash -c 'source env_{self.env_name }/bin/activate  && pip freeze > requirements.txt'"
            )
            print(
                self.notice
                + f'{self.colors.SKY_BLUE}Successfully updated the file "requirements.txt"{self.colors.ENDC}'
            )



""" Code module for generating content files """


class Commands:
    def __init__(self, **kwargs) -> None:
        self.kwargs = kwargs

    def get_main_py(self):
        return """
def main():
    pass


if __name__ == "__main__":
    main()
        """

    def get_main_py(self):
        return """
def main():
    pass


if __name__ == "__main__":
    main()
        """

    def get_readme_md(self):
        return """
# {}
A brief and descriptive title for your project.

## Description

A detailed description of the project, including its purpose, features, and any other relevant information.

## Getting Started

```
git clone https://github.com/<User Name Github>/{}.git

cd {}

```

## Installation

```
# create virtualenv auto name
fenv onlyenv

# install package in requirements.txt
fenv install

```

## Usage

Instructions on how to use the project, including any usage examples and screenshots.

## Tree
<!--- Start Tree --->
```bash
.
└── {}/
{}
```
<!--- End Tree --->

## Contributing

If you would like to contribute to the project, include a section on how to do so, including any guidelines and best practices.

## License

Include information about the license used for the project, such as the name of the license (e.g. MIT, Apache 2.0, etc.) and a link to the license text.

"""

    def get_requirements_txt(self):
        return """
black
fenv
"""

    def get_update_tree_path(self):
        return """
<!--- Start Tree --->
```bash
.
└── {}/
{}
```
"""



from setuptools import setup, find_packages
from dotenv import load_dotenv

load_dotenv()
import codecs
import os

import fileinput

new_version = os.getenv("FENV_VERSION")
file_path = os.path.abspath(os.path.dirname(__file__)) + "/fenv/fenv.py"

# Replace the first line of the file with the new version string
with fileinput.FileInput(file_path, inplace=True, encoding="utf-8") as file:
    for line_number, line in enumerate(file):
        if line_number == 0:
            print(f"version: str = '{new_version}'")
        else:
            print(line.rstrip())

# Print the original content of the file
with open(file_path, "r", encoding="utf-8") as file:
    original_content = file.read()
    print(original_content)

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "readme.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = os.getenv("FENV_VERSION")
DESCRIPTION = (
    "Generate a folder, establish a virtual environment with a single command."
)
LONG_DESCRIPTION = "Generate a folder, establish a virtual environment, and simultaneously create the essential basic Python files, all with a single command"

# Setting up
setup(
    name="Fenv",
    version=VERSION,
    author="wk18k (watchakorn-18k)",
    author_email="<porton555@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=["virtualenv"],
    entry_points="""
    [console_scripts]
    fenv=fenv.fenv:main
    """,
    keywords=["python", "virtualenv", "create file", "create folder", "fenv", "wk-18k"],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
    ],
)
